"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const prebuildTheme_1 = require("./theme/prebuildTheme");
const fallback_1 = require("./theme/fallback");
const buildMetaTree_1 = require("./buildMetaTree/buildMetaTree");
const render_1 = require("../meta/render/render");
const defaultSettings_1 = require("./defaultSettings");
/**
  * Utility to print pretty formatted tree-representation of any javacript data.

 * This class represens basic API and works both ways:
 * - as a constructable via `new Pretty(themeStack)` instance,
 * - or as a static class with oneshot functions `Pretty.print` and `Pretty.stringify`
 *
 * @export
 * @class Pretty
 */
class Pretty {
    constructor(themeStack) {
        this.theme = prebuildTheme_1.prebuildTheme(themeStack || [], fallback_1.fallback);
    }
    /**
     * Prints pretty formatted tree-representation of any javacript data. Use options
     * to customize output.
     *
     * @remarks
     *
     * Use `options.logLineCallback` for custom `console.log` handler, or
     * `pretty.stringifyTree` to render tree to a `string`.
     *
     * @remarks
     *
     * Cycle-referencing is allowed, while capping output
     * with finite `options.maxLevel` and `options.maxItemsPerLevel`.
     *
     * @param {*} tree Any javacript data
     * @param {*} [rootKey] Optional key name of tree root
     * @param {printTreeOptionsT} [options]
     * @memberof Pretty
     */
    print(tree, rootKey, options) {
        const settings = Object.assign({}, defaultSettings_1.defaultSettings, options);
        const metaTree = buildMetaTree_1.buildMetaTree(tree, rootKey, this.theme, settings);
        render_1.renderMetaNode(metaTree, settings.maxLineWidth, settings.renderer, settings.logLineCallback);
    }
    /**
     * Renders pretty formatted tree-representation of any javacript data to string.
     * Use options to customize output.
     *
     * @remarks
     *
     * Works same as `pretty.print`, but use predefined `logLineCallback`
     * for dumping output to `string`. Consider Using `pretty.print` if
     * you need a per-line callback API, or if you just want a simple oneliner
     * with default `options.logLineCallback`
     *
     * @remarks
     *
     * Cycle-referencing is allowed, while capping output
     * with finite `options.maxLevel` and `options.maxItemsPerLevel`.
     *
     *
     * @param {*} tree Any javacript data
     * @param {*} [rootKey] Optional key name of tree root
     * @param {stringifyTreeOptionsT} [options]
     * @returns {string}
     * @memberof Pretty
     */
    stringify(tree, rootKey, options) {
        let stringBuffer = '';
        const printTreeOptions = Object.assign({}, options || {}, {
            logLineCallback(line, trailingEol) { stringBuffer += line + trailingEol; },
        });
        this.print(tree, rootKey, printTreeOptions);
        return stringBuffer;
    }
    /**
     * Prints pretty formatted tree-representation of any javacript data. Use options
     * to customize output.
     *
     * @remarks
     *
     * Static "oneshot" version of `pretty.print`.
     * Easier to use, because no need to create `new Pretty()`,
     * but it prebuilds theme **each time** it starts,
     * while long-living `new Pretty()` keep prebuilded theme
     * and reused it.
     *
     * @remarks
     *
     * Options are `printTreeOptionsT` plus `themeStack` property. Which is intended for
     * `Pretty` constructor to create short-living `new Pretty(themeStack)`.
     *
     * @remarks
     *
     * Use `options.logLineCallback` for custom `console.log` handler, or
     * `pretty.stringifyTree` to render tree to a `string`.
     *
     * @remarks
     *
     * Cycle-referencing is allowed, while capping output
     * with finite `options.maxLevel` and `options.maxItemsPerLevel`.
     *
     * @static
     * @param {*} tree Any javacript data
     * @param {*} [rootKey] Optional key name of tree root
     * @param {oneshotPrintTreeOptionsT} [options] same as `printTreeOptionsT` but with `themeStack`
     * @memberof Pretty
     */
    static print(tree, rootKey, options) {
        const settings = Object.assign({}, defaultSettings_1.defaultSettings, options);
        const oneshotPretty = new Pretty(settings.themeStack);
        oneshotPretty.print(tree, rootKey, settings);
    }
    /**
     * Renders pretty formatted tree-representation of any javacript data to string.
     * Use options to customize output.
     *
     * @remarks
     *
     * Static "oneshot" version of `pretty.print`.
     * Easier to use, because no need to create `new Pretty()`,
     * but it prebuilds theme **each time** it starts,
     * while long-living `new Pretty()` keep prebuilded theme
     * and reused it.
     *
     * @remarks
     *
     * Options are `stringifyTreeOptionsT` plus `themeStack` property. Which is intended for
     * `Pretty` constructor to create short-living `new Pretty(themeStack)`.
     *
     * @remarks
     *
     * Works same as `pretty.print`, but use predefined `logLineCallback`
     * for dumping output to `string`. Consider Using `pretty.print` if
     * you need a per-line callback API, or if you just want a simple oneliner
     * with default `options.logLineCallback`.
     *
     * @remarks
     *
     * Cycle-referencing is allowed, while capping output
     * with finite `options.maxLevel` and `options.maxItemsPerLevel`.
     *
     *
     * @static
     * @param {*} tree Any javacript data
     * @param {*} [rootKey] Optional key name of tree root
     * @param {oneshotStringifyTreeOptionsT} [options] same as `stringifyTreeOptionsT` but with `themeStack`
     * @returns {string}
     * @memberof Pretty
     */
    static stringify(tree, rootKey, options) {
        const settings = Object.assign({}, defaultSettings_1.defaultSettings, options);
        const oneshotPretty = new Pretty(settings.themeStack);
        return oneshotPretty.stringify(tree, rootKey, settings);
    }
}
exports.Pretty = Pretty;
//# sourceMappingURL=pretty.js.map